==> kmuddy-0.7.1.tar.gz: kmuddy-0.7.1/kmuddy/ctextprocessor.cpp <==
/***************************************************************************
                          ctextprocessor.cpp  -  text processor
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po oct 14 2002
    copyright            : (C) 2002-2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QT_CLEAN_NAMESPACE
#define QT_CLEAN_NAMESPACE
#endif

#include <config.h>

#include "ctextprocessor.h"

#include <knotifyclient.h>

#include <qtimer.h>

#include "cconnection.h"
#include "cconnprefs.h"
#include "cconsole.h"
#include "cpluginmanager.h"
#include "crunninglist.h"
#include "csession.h"
#include "ctextchunk.h"
#include "ctriggerlist.h"
#include "cvariablelist.h"
#include "cwindowlist.h"

#ifdef HAVE_MXP
#include "cmxpmanager.h"
#endif //HAVE_MXP

cTextProcessor::cTextProcessor ()
{
  chunk = 0;

  flush ();
  gotprompt = false;

  //some defaults...
  fg = gray;
  bg = black;
  attrib = 0;

  elapsedticks = 0;
  pdtimer = new QTimer;
  connect (pdtimer, SIGNAL (timeout ()), this, SLOT (timeout ()));
  pdtimer->start (250);
}

cTextProcessor::~cTextProcessor ()
{
  pdtimer->stop ();
  delete pdtimer;
}

void cTextProcessor::preprocess (const QString &text)
{
  //let plug-ins preprocess the text first
  QString txt = text;  //drop the const flag
  pluginmanager->preprocessInput (sess, txt);

  cConnPrefs *cp = sess->connection()->getConnPrefs();
  if (cp)
  {
    cRunningList *rl = cp->runningList();
    if (rl->havePreprocessorScript ())
    {
      //send the text to preprocessor, if we have one
      rl->sendToPreprocessor (txt);
      return;
    }
  }
  //else we do nothing
  emit preprocessed (txt);
}

void cTextProcessor::preprocessorOutput (const QString &text)
{
  //text should be in format <number> <text>
  //where number is 1 or 2, 1 means standard text (one line), 2 means prompt
  if (text.isEmpty()) return;
  char ch = text[0].latin1();
  QString t = text;
  t.remove (0, 2);
  if (ch == '1')
  {
    //if a prompt is the last thing displayed, we send a newline, so that
    //the text coming from preprocessor gets to the new line. Note that
    //gotprompt is set to false if the user sends some command, so we needn't
    //worry about that
    if (gotprompt)
      emit preprocessed ("\n");

    //send the line
    emit preprocessed (t + "\n");
  }
  if (ch == '2')
  {
    emit preprocessed (t);
    //... processing takes place ...
    //... we regain control when everything is set for the following stuff:
    //part of receivedGA():
    if (!gotprompt)
    {
      if (chunk)
      {
        pluginmanager->passPrompt (sess, chunk);
        emit receivedPrompt (chunk->plainText());
        emit displayPrompt (chunk);
        delete chunk;
        chunk = 0;
      }
      gotprompt = true;
    }
  }
}

void cTextProcessor::parseText (const QString &text)
{
  //TODO: if MXP is on, pass the text to MXP parser...
#ifdef HAVE_MXP
  if (sess->mxpmanager()->isMXPActive ())
  {
    sess->mxpmanager()->processText (text);
    return;
  }
#endif
  //this will only be run if MXP is off...
  QString t;
  int l = text.length ();
  for (int i = 0; i < l; i++)
  {
    QChar ch = text[i];
    if (ch == '\n')
    {
      if (t.length())
        emit plainText (t);
      gotNewLine ();
      t = "";
    }
    else
      t += text[i];
  }

  if (t.length())
    emit plainText (t);
}

void cTextProcessor::sendingCommand ()
{
  //we can receive a prompt again, even if no newline arrives
  //this is to support things like 'Name:', then 'Password:', but
  //no newline between them (as that comes from the user)
  gotprompt = false;
}

void cTextProcessor::gotNewText (const QString &text)
{
  //if we already have a prompt, then this will go to a new line
  //this is to prevent multiple prompts on one line and similar things
  // *** CODE DISABLED FOR NOW ***
  //if (gotprompt)
  //  gotNewLine ();

  if (chunk == 0)
    createChunk ();

  //reset tick-counter
  elapsedticks = 0;

  int len = text.length();
  QString txt;
  for (int i = 0; i < len; i++)
  {
    if (text[i] == '\t')  //TAB character
      //add 8 spaces - not perfect, but hey ;)
      for (int i = 0; i < 8; i++)
        txt += QChar (' ');
    else if (text[i] == '\a')  //beep
      KNotifyClient::beep ();
    else
      txt += text[i];
  }

  //create the text chunk an'all :)
  chunkText *cht = new chunkText;
  cht->setText (txt);
  chunk->appendEntry (cht);

  //prompt detection (only if no prompt has been recvd since end of line)
  if ((!txt.isEmpty()) && (!gotprompt))
  {
    haveprompt = false;

    cConnPrefs *cp = sess->connection()->getConnPrefs ();
    //if cp is NULL, we're not connected using a profile => no triggers
    if (cp != NULL)
    {
      cTriggerList *triggers = cp->triggerList();
      if (triggers != NULL)
      {
        //only prompt-detecting triggers will be executed here
        triggers->setDetectingPrompt (true);
        triggers->matchString (chunk);
      }
      if (haveprompt)
        //if triggers say that this is a prompt, then we behave as if
        //telnet-GA was received
        receivedGA ();
    }
  }
}

void cTextProcessor::gotFgColor (QColor color)
{
  if (chunk == 0)
    createChunk ();

  chunkFg *chfg = new chunkFg;
  chfg->setFg (color);
  chunk->appendEntry (chfg);

  //remember the color...
  fg = color;
}

void cTextProcessor::gotBgColor (QColor color)
{
  if (chunk == 0)
    createChunk ();

  chunkBg *chbg = new chunkBg;
  chbg->setBg (color);
  chunk->appendEntry (chbg);

  //remember the color...
  bg = color;
}

void cTextProcessor::gotAttrib (int a)
{
  if (chunk == 0)
    createChunk ();

  chunkAttrib *chatt = new chunkAttrib;
  chatt->setAttrib (a);
  chunk->appendEntry (chatt);

  //remember the attribute...
  attrib = a;
}

void cTextProcessor::gotALink (const QString &name, const QString &url, const QString &text,
    const QString &hint)
{
  if (chunk == 0)
    createChunk ();

  chunkLink *chl = new chunkLink;
  chl->setIsCommand (false);
  chl->setIsMenu (false);
  chl->setName (name);
  chl->setTarget (url);
  chl->setText (text);
  chl->setHint (hint);
  
  chunk->appendEntry (chl);
}

void cTextProcessor::gotSENDLink (const QString &name, const QString &command,
    const QString &text, const QString &hint, bool toprompt, bool ismenu)
{
  if (chunk == 0)
    createChunk ();

  chunkLink *chl = new chunkLink;
  chl->setIsCommand (true);
  chl->setName (name);
  chl->setTarget (command);
  chl->setText (text);
  chl->setHint (hint);
  chl->setToPrompt (toprompt);
  chl->setIsMenu (ismenu);
  
  //parse menu information of needed
  if (ismenu)
    chl->parseMenu();

  chunk->appendEntry (chl);
}

void cTextProcessor::createChunk ()
{
  chunk = new cTextChunk (sess->console());
  chunkStart startattr;
  startattr.startpos = 0;
  startattr.fg = fg;
  startattr.bg = bg;
  startattr.attrib = attrib;
  chunk->setStartAttr (startattr);
}

void cTextProcessor::gotNewLine ()
{
  cConnPrefs *cp = sess->connection()->getConnPrefs ();

  if (!chunk)  //empty line - create an empty one
    createChunk ();

  //inform plug-ins - phase 1 (before triggers)
  pluginmanager->passInput (sess, 1, chunk, false);

  //is a gagging trigger gets activated, it will call gagLine(), which will
  //set this flag
  gag = false;
  //the same for primary gag, which is set by window-redirection triggers
  primarygag = false;
  //also clear window name
  wname = QString::null;

  //first call trigger comparison/execution...
  //if cp is NULL, we're not connected using a profile => no triggers
  if (cp != NULL)
  {
    cTriggerList *triggers = cp->triggerList();
    if (triggers != NULL)
    {
      triggers->setDetectingPrompt (false);
      triggers->matchString (chunk);
    }
  }

  //inform plug-ins - phase 2 (after triggers)
  pluginmanager->passInput (sess, 2, chunk, gag);

  QString plainText = chunk->plainText();

  //now emit some signals, if the line shouldn't be gagged...
  //the cConsole class is connected here, amongst others
  if (!gag)
  {
    if (wname)
    {
      if (cp)
      {
        cWindowList *wlist = cp->windowList();
        //send the text to the output window, if it exists
        if (wlist->exists (wname))
          wlist->textToWindow (wname, chunk);
        else
          //if the window doesn't exist, display the text in the main window, even if it would
          //normally be gagged
          //this is to prevent loss of information in case that the user removes some window,
          //but keeps triggers that use that window
          primarygag = false;
      }
    }
    if (!primarygag)
    {
      emit displayLine (chunk);
      delete chunk;
      chunk = 0;
    }
  }
  //textLine is emitted even if line is gagged - this means that scripts will receive the line
  //even if it is gagged... Transcript uses displayLine so gagged lines won't be written into
  //session transcript
  emit textLine (plainText);


  //clear pointer to old line data - it is now either deleted or managed by cConsole
  chunk = 0;

  //increase line counter (even if line was gagged), and set $line variable in variableList
  lines++;

  if (cp != NULL)
  {
    cVariableList *vl = cp->variableList();
    QString str_lines;
    str_lines.setNum(lines);
    vl->set (QString("line"), str_lines);
  } // endif we have a connection profile

  //we can receive prompt again
  gotprompt = false;
}

void cTextProcessor::flush ()
{
  lines = 0;

  delete chunk;
  chunk = 0;

  gag = primarygag = false;
  haveprompt = gotprompt = false;
}

void cTextProcessor::receivedGA ()
{
  //ignore GA signals if a preprocessor script exists
  //just inform the running script that GA is here
  cConnPrefs *cp = sess->connection()->getConnPrefs();
  if (cp)
  {
    cRunningList *rl = cp->runningList();
    if (rl->havePreprocessorScript ())
    {
      rl->sendToPreprocessor ("\x01\n");
      return;
    }
  }

  if (!gotprompt)  //avoid 2+ prompts on one line
  {
    if (chunk)
    {
      pluginmanager->passPrompt (sess, chunk);

      emit receivedPrompt (chunk->plainText());
      emit displayPrompt (chunk);

      //if cp is NULL, we're not connected using a profile => no triggers
      if (cp != NULL)
      {
        cTriggerList *triggers = cp->triggerList();
        if (triggers != NULL)
        {
          triggers->setDetectingPrompt (false);
          triggers->matchString (chunk);
        }
      } // endif we have a profile
    
      delete chunk;
      chunk = 0;
    }

    gotprompt = true;

    //increase line counter (even if line was gagged), and set $line variable in variableList
    lines++;

    if (cp != NULL)
    {
      cVariableList *vl = cp->variableList();
      QString str_lines;
      str_lines.setNum(lines);
      vl->set (QString("line"), str_lines);
    } // endif we have a connection profile


  }
}

void cTextProcessor::moreTextHere ()
{
  if (chunk && !chunk->entries().empty())
    emit showPartialLine (chunk->plainText());
}

void cTextProcessor::recolorize (list<colorChange> &changes)
{
  if (chunk)
    chunk->applyColorChanges (changes);
}

void cTextProcessor::timeout ()
{
  //If some part of the line is waiting, and no prompt has been received
  //so far, we increase our counter. If the counter reaches 4 and no more
  //text arrives, we assume that the waiting text is a prompt and we
  //treat it as such.

  if ((!gotprompt) && chunk && (!chunk->entries().empty()))
    elapsedticks++;
  if (elapsedticks == 4)  //it's probably a prompt
    receivedGA ();
}

==> kmuddy-0.8.tar.gz: kmuddy-0.8/kmuddy/ctextprocessor.cpp <==
/***************************************************************************
                          ctextprocessor.cpp  -  text processor
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po oct 14 2002
    copyright            : (C) 2002-2003 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QT_CLEAN_NAMESPACE
#define QT_CLEAN_NAMESPACE
#endif

#include <config.h>

#include "ctextprocessor.h"

#include "cansiparser.h"
#include "cconsole.h"
#include "coutput.h"
#include "ctextchunk.h"
#include "ctriggerlist.h"
#include "cvariablelist.h"
#include "cwindowlist.h"

#ifdef HAVE_MXP
#include "cmxpmanager.h"
#endif //HAVE_MXP

#include <knotifyclient.h>
#include <qtimer.h>

cTextProcessor::cTextProcessor (int sess) : cActionBase ("textproc", sess)
{
  chunk = 0;

  flush ();
  gotprompt = false;

  //some defaults...
  fg = gray;
  bg = black;
  attrib = 0;

  elapsedticks = 0;
  pdtimer = new QTimer;
  connect (pdtimer, SIGNAL (timeout ()), this, SLOT (timeout ()));
  pdtimer->start (250);
  
  // some events ahd stuff ...
  addEventHandler ("data-received", 50, PT_STRING);
  addEventHandler ("received-ga", 50, PT_NOTHING);
  addEventHandler ("connected", 100, PT_NOTHING);
  addEventHandler ("command-sent", 50, PT_STRING);
  
  // Both ANSI parser and MXP parser must already exist when this is being created
  // That is due to the following code ...

  cANSIParser *ap = dynamic_cast<cANSIParser *>(object ("ansiparser", sess));
#ifdef HAVE_MXP
  cMXPManager *mxpmanager = dynamic_cast<cMXPManager *>(object ("mxpmanager", sess));
  connect (mxpmanager, SIGNAL (gotNewText (const QString &)),
      ap, SLOT (parseText (const QString &)));
  connect (mxpmanager, SIGNAL (gotNewLine ()), this, SLOT (gotNewLine ()));
  connect (mxpmanager, SIGNAL (gotFgColor (QColor)), this, SLOT (gotFgColor (QColor)));
  connect (mxpmanager, SIGNAL (gotBgColor (QColor)), this, SLOT (gotBgColor (QColor)));
  connect (mxpmanager, SIGNAL (gotAttrib (int)), this, SLOT (gotAttrib (int)));
  connect (mxpmanager, SIGNAL (gotALink (const QString &, const QString &, const QString &,
      const QString &)), this, SLOT (gotALink (const QString &, const QString &,
      const QString &, const QString &)));
  connect (mxpmanager, SIGNAL (gotSENDLink (const QString &, const QString &, const QString &,
      const QString &, bool, bool)), this, SLOT (gotSENDLink (const QString &,
      const QString &, const QString &, const QString &, bool, bool)));
  connect (mxpmanager, SIGNAL (gotExpire (const QString &)), this,
      SLOT (gotExpire (const QString &)));
  
#endif
  connect (this, SIGNAL (plainText (const QString &)), ap, SLOT (parseText (const QString &)));
  connect (ap, SIGNAL (fgColor (QColor)), this, SLOT (gotFgColor (QColor)));
  connect (ap, SIGNAL (bgColor (QColor)), this, SLOT (gotBgColor (QColor)));
  connect (ap, SIGNAL (attrib (int)), this, SLOT (gotAttrib (int)));
  connect (ap, SIGNAL (plainText (const QString &)), this, SLOT (gotNewText (const QString &)));
}

cTextProcessor::~cTextProcessor ()
{
  removeEventHandler ("data-received");
  removeEventHandler ("received-ga");
  removeEventHandler ("connected");
  removeEventHandler ("command-sent");

  pdtimer->stop ();
  delete pdtimer;
}

void cTextProcessor::eventHandler (QString event, int, QString &par1,
    const QString &)
{
  if (event == "data-received")
    parseText (par1);
  else if (event == "command-sent")
    sendingCommand ();
}

void cTextProcessor::eventHandler (QString event, int)
{
  if (event == "received-ga")
    receivedGA ();
  else if (event == "connected")
    flush ();
  else if (event == "text-here")
    moreTextHere ();
}

void cTextProcessor::parseText (const QString &text)
{
  // TODO: Both MXP and ANSI parsing should be done via events, not like this ...
  // This is so that we can easily add new protocols and so on ...
#ifdef HAVE_MXP
  //if MXP is on, pass the text to MXP parser...
  cMXPManager *mxpmanager = dynamic_cast<cMXPManager *>(object ("mxpmanager", sess()));
  if (mxpmanager->isMXPActive ())
  {
    mxpmanager->processText (text);
    return;
  }
#endif
  //this will only be run if MXP is off...
  QString t;
  int l = text.length ();
  for (int i = 0; i < l; i++)
  {
    QChar ch = text[i];
    if (ch == '\n')
    {
      if (t.length())
        emit plainText (t);
      gotNewLine ();
      t = "";
    }
    else
      t += text[i];
  }

  if (t.length())
    emit plainText (t);
}

void cTextProcessor::sendingCommand ()
{
  //we can receive a prompt again, even if no newline arrives
  //this is to support things like 'Name:', then 'Password:', but
  //no newline between them (as that comes from the user)
  gotprompt = false;
}

void cTextProcessor::gotNewText (const QString &text)
{
  //if we already have a prompt, then this will go to a new line
  //this is to prevent multiple prompts on one line and similar things
  // *** CODE DISABLED FOR NOW ***
  //if (gotprompt)
  //  gotNewLine ();

  if (chunk == 0)
    createChunk ();

  //reset tick-counter
  elapsedticks = 0;

  int len = text.length();
  QString txt;
  for (int i = 0; i < len; i++)
  {
    if (text[i] == '\t')  //TAB character
      //add 8 spaces - not perfect, but hey ;)
      for (int i = 0; i < 8; i++)
        txt += QChar (' ');
    else if (text[i] == '\a')  //beep
      KNotifyClient::beep ();
    else
      txt += text[i];
  }

  //create the text chunk an'all :)
  chunkText *cht = new chunkText;
  cht->setText (txt);
  chunk->appendEntry (cht);

  //prompt detection (only if no prompt has been recvd since end of line)
  if ((!txt.isEmpty()) && (!gotprompt))
  {
    haveprompt = false;

    cTriggerList *triggers = dynamic_cast<cTriggerList *>(object ("triggerlist"));
    //if cp is NULL, we're not connected using a profile => no triggers
    if (triggers != 0)
    {
      //only prompt-detecting triggers will be executed here
      triggers->setDetectingPrompt (true);
      triggers->matchString (chunk);
    }
    if (haveprompt)
      //if triggers say that this is a prompt, then we behave as if
      //telnet-GA was received
      receivedGA ();
  }
}

void cTextProcessor::gotFgColor (QColor color)
{
  if (chunk == 0)
    createChunk ();

  chunkFg *chfg = new chunkFg;
  chfg->setFg (color);
  chunk->appendEntry (chfg);

  //remember the color...
  fg = color;
}

void cTextProcessor::gotBgColor (QColor color)
{
  if (chunk == 0)
    createChunk ();

  chunkBg *chbg = new chunkBg;
  chbg->setBg (color);
  chunk->appendEntry (chbg);

  //remember the color...
  bg = color;
}

void cTextProcessor::gotAttrib (int a)
{
  if (chunk == 0)
    createChunk ();

  chunkAttrib *chatt = new chunkAttrib;
  chatt->setAttrib (a);
  chunk->appendEntry (chatt);

  //remember the attribute...
  attrib = a;
}

void cTextProcessor::gotALink (const QString &name, const QString &url, const QString &text,
    const QString &hint)
{
  if (chunk == 0)
    createChunk ();

  chunkLink *chl = new chunkLink;
  chl->setIsCommand (false);
  chl->setIsMenu (false);
  chl->setName (name);
  chl->setTarget (url);
  chl->setText (text);
  chl->setHint (hint);
  
  chunk->appendEntry (chl);
}

void cTextProcessor::gotSENDLink (const QString &name, const QString &command,
    const QString &text, const QString &hint, bool toprompt, bool ismenu)
{
  if (chunk == 0)
    createChunk ();

  chunkLink *chl = new chunkLink;
  chl->setIsCommand (true);
  chl->setName (name);
  chl->setTarget (command);
  chl->setText (text);
  chl->setHint (hint);
  chl->setToPrompt (toprompt);
  chl->setIsMenu (ismenu);
  
  //parse menu information of needed
  if (ismenu)
    chl->parseMenu();

  chunk->appendEntry (chl);
}

void cTextProcessor::gotExpire (const QString &name)
{
  cConsole *console = (dynamic_cast<cOutput *>(object ("output")))->console();
  console->expireNamedLinks (name);
}

void cTextProcessor::createChunk ()
{
  chunk = new cTextChunk ((dynamic_cast<cOutput *>(object ("output")))->console());
  chunkStart startattr;
  startattr.startpos = 0;
  startattr.fg = fg;
  startattr.bg = bg;
  startattr.attrib = attrib;
  chunk->setStartAttr (startattr);
}

void cTextProcessor::gotNewLine ()
{
  if (!chunk)  //empty line - create an empty one
    createChunk ();

  //inform plug-ins - phase 1 (before triggers)
  invokeEvent ("got-line", sess(), chunk);

  //is a gagging trigger gets activated, it will call gagLine(), which will
  //set this flag
  gag = false;
  //the same for primary gag, which is set by window-redirection triggers
  primarygag = false;
  //also clear window name
  wname = QString::null;

  //first call trigger comparison/execution...

  cTriggerList *triggers = dynamic_cast<cTriggerList *>(object ("triggerlist"));
  if (triggers)
  {
    triggers->setDetectingPrompt (false);
    triggers->matchString (chunk);
  }

  //inform plug-ins - phase 2 (after triggers)
  if (gag) invokeEvent ("will-gag", sess());
  invokeEvent ("got-line-after-triggers", sess(), chunk);

  QString plainText = chunk->plainText();

  //now emit some events, if the line shouldn't be gagged...
  //the cOutput class is connected here, amongst others
  if (!gag)
  {
    if (wname)
    {
      cWindowList *wlist = dynamic_cast<cWindowList *>(object ("windowlist"));
      if (wlist) {
        //send the text to the output window, if it exists
        if (wlist->exists (wname))
          wlist->textToWindow (wname, chunk);
        else
          //if the window doesn't exist, display the text in the main window, even if it would
          //normally be gagged
          //this is to prevent loss of information in case that the user removes some window,
          //but keeps triggers that use that window
          primarygag = false;
      }
    }
    if (!primarygag)
      invokeEvent ("display-line", sess(), chunk);
  }
  //got-line is emitted even if line is gagged - this means that scripts will receive the line
  //even if it is gagged...
  invokeEvent ("got-line", sess(), plainText);

  delete chunk;
  chunk = 0;

  //increase line counter (even if line was gagged), and set $line variable in variableList
  lines++;

  cVariableList *vl = dynamic_cast<cVariableList *>(object ("variablelist"));
  if (vl) {
    QString str_lines;
    str_lines.setNum(lines);
    vl->set (QString("line"), str_lines);
  }

  //we can receive prompt again
  gotprompt = false;
}

void cTextProcessor::flush ()
{
  lines = 0;

  delete chunk;
  chunk = 0;

  gag = primarygag = false;
  haveprompt = gotprompt = false;
}

void cTextProcessor::receivedGA ()
{
  if (!gotprompt)  //avoid 2+ prompts on one line
  {
    if (chunk)
    {
      invokeEvent ("got-prompt", sess(), chunk->plainText());
      invokeEvent ("got-prompt", sess(), chunk);

      cTriggerList *triggers = dynamic_cast<cTriggerList *>(object ("triggerlist"));
      if (triggers)
      {
        triggers->setDetectingPrompt (false);
        triggers->matchString (chunk);
        invokeEvent ("got-prompt-after-triggers", sess(), chunk);
      }
      invokeEvent ("display-prompt", sess(), chunk);
    
      delete chunk;
      chunk = 0;
    }

    gotprompt = true;

    //increase line counter (even if line was gagged), and set $line variable in variableList
    lines++;

    cVariableList *vl = dynamic_cast<cVariableList *>(object ("variablelist"));
    if (vl)
    {
      QString str_lines;
      str_lines.setNum(lines);
      vl->set (QString("line"), str_lines);
    }
  }
}

void cTextProcessor::moreTextHere ()
{
  if (chunk && !chunk->entries().empty())
   invokeEvent ("partial-line", sess(), chunk->plainText());
}

void cTextProcessor::recolorize (list<colorChange> &changes)
{
  if (chunk)
    chunk->applyColorChanges (changes);
}

void cTextProcessor::timeout ()
{
  //If some part of the line is waiting, and no prompt has been received
  //so far, we increase our counter. If the counter reaches 4 and no more
  //text arrives, we assume that the waiting text is a prompt and we
  //treat it as such.

  if ((!gotprompt) && chunk && (!chunk->entries().empty()))
    elapsedticks++;
  if (elapsedticks == 4)  //it's probably a prompt
    receivedGA ();
}
